#include "ICellMLToRDFTree.h"
#include <nsIRDFLiteral.h>
#include <nsCOMPtr.h>
#include <nsIRDFService.h>
#include <nsIRDFContainerUtils.h>
#include <nsClassHashtable.h>
#include <nsDataHashtable.h>
#include <nsInterfaceHashtable.h>

class CellMLToRDFTree;
typedef void (*rdf_populate_t)(CellMLToRDFTree*, nsISupports*,
                               nsIRDFContainer*);

class CellMLToRDFTree
  : public ICellMLToRDFTree
{
public:
  CellMLToRDFTree();
  ~CellMLToRDFTree();

  NS_DECL_ISUPPORTS;
  NS_DECL_ICELLMLTORDFTREE;

private:
  class ResourceAssociation
  {
  public:
    ResourceAssociation(const char* aType,
                        nsISupports* aElement,
                        CellMLToRDFTree* aCellMLToRDFTree,
                        nsIRDFContainer* aSeq,
                        rdf_populate_t aPopulate);

    const char* mType;
    nsCOMPtr<nsISupports> mElement;
    nsCOMPtr<nsIRDFContainer> mSeq;
    CellMLToRDFTree* mCellMLToRDFTree;
    rdf_populate_t mPopulate;
  };

public:
  nsCOMPtr<nsIRDFDataSource> mDatasource;
  nsClassHashtable<nsVoidPtrHashKey,ResourceAssociation> mHash;
  nsDataHashtable<nsCStringHashKey,int> mOpenSet;
  nsInterfaceHashtable<nsCStringHashKey, nsIRDFResource> mResourceHash;
  static void EnsureLiterals();
  static bool mSetupStatic;
  static nsIRDFResource * mRootResource, * mIvRootResource, * mTypeResource,
    * mPropResource, * mValueResource, * mValuePropResource, * mOpenResource,
    * mPlaceholderResource;
  static nsIRDFLiteral * mTrueLiteral, * mNumberLiteral, * mModelLiteral,
    * mComponentsLiteral, * mGroupLiteral, * mGroupsLiteral, * mConnectionsLiteral,
    * mConnectionLiteral, * mUnitsLiteral, * mImportsLiteral, * mVariablesLiteral,
    * mRolesLiteral, * mPubIntLiteral, * mPrivIntLiteral, * mBaseLiteral,
    * mDerivedLiteral, * mPrefixLiteral, * mExponentLiteral, * mOffsetLiteral,
    * mMultiplierLiteral, * mUnitLiteral, * mReversibleLiteral,
    * mIrreversibleLiteral, * mReactionLiteral, * mVariableRefLiteral,
    * mReactantLiteral, * mProductLiteral, * mRateLiteral, * mCatalystLiteral,
    * mActivatorLiteral, * mInhibitorLiteral, * mModifierLiteral,
    * mDirectionLiteral, * mForwardLiteral, * mReverseLiteral, * mBothLiteral,
    * mStoichiometryLiteral, * mDeltaVariableLiteral,
    * mRelationshipRefLiteral, * mNameLiteral, * mNamespaceLiteral,
    * mComponentRefLiteral, * mComponent1Literal, * mComponent2Literal,
    * mVariable1Literal, * mVariable2Literal, * mVariableMappingLiteral,
    * mMapVariablesLiteral, * mMathLiteral, * mEquationLiteral;
  static nsIRDFLiteral * mInterfaceIdLiteral[3];
  static nsCOMPtr<nsIRDFService> mRDFService;
  static nsCOMPtr<nsIRDFContainerUtils> mContUtils;
  static nsCOMPtr<nsIRDFDataSource> mPersistSource;

  void ExpandResourceInTree(nsIRDFResource* aResource);
  already_AddRefed<nsIRDFResource> GetAssociatedResource
    (const char* aType, nsISupports* aElement,
     rdf_populate_t aPopulate, PRBool aAlwaysOpen=PR_FALSE);
  void PopulateModelIntoSequences
  (
   cellml_api_IModel* aModel,
   nsIRDFContainer* aRootseq
  );
  void PopulateComponentsIntoSequences
  (
   cellml_api_ICellMLComponentIterator* aCompIt,
   nsIRDFContainer* aCompSeq
  );
  void PopulateGroupsIntoSequences
  (
   cellml_api_IGroupIterator* aGroupIt,
   nsIRDFContainer* aGroupSeq
  );
  void PopulateConnectionsIntoSequences
  (
   cellml_api_IConnectionIterator* aConnIt,
   nsIRDFContainer* aConnSeq
  );
  void PopulateUnitsIntoSequences
  (
   cellml_api_IUnitsIterator* aUnitsIt,
   nsIRDFContainer* aUnitsSeq
  );
  void PopulateImportsIntoSequences
  (
   cellml_api_ICellMLImportIterator* aCompIt,
   nsIRDFContainer* aImpSeq
  );
  void PopulateVariablesIntoSequences
  (
   cellml_api_ICellMLVariableIterator* aVarIt,
   nsIRDFContainer* aVarSeq,
   PRBool aIgnoreNonIV = PR_FALSE
  );
  void PopulateReactionsIntoSequences
  (
   cellml_api_IReactionIterator* aReactIt,
   nsIRDFContainer* aReactSeq
  );
  void PopulateRelationshipRefsIntoSequences
  (
   cellml_api_IRelationshipRefIterator* aRRIt,
   nsIRDFContainer* aRRSeq
  );
  void PopulateComponentRefsIntoSequences
  (
   cellml_api_IComponentRefIterator* aCRIt,
   nsIRDFContainer* aCRSeq
  );
  void PopulateMapVariablesIntoSequences
  (
   cellml_api_IMapVariablesIterator* aMVIt,
   nsIRDFContainer* aMVSeq
  );
  void PopulateUnitIntoSequences
  (
   cellml_api_IUnitIterator* aUnIt,
   nsIRDFContainer* aUnitSeq
  );
  void PopulateImportComponentsIntoSequences
  (
   cellml_api_IImportComponentIterator* aMVIt,
   nsIRDFContainer* aICSeq
  );
  void PopulateImportUnitsIntoSequences
  (
   cellml_api_IImportUnitsIterator* aIUIt,
   nsIRDFContainer* aIUSeq
  );
  void PopulateVariableRefsIntoSequences
  (
   cellml_api_IVariableRefIterator* aVRIt,
   nsIRDFContainer* aVRSeq
  );
  void PopulateRolesIntoSequences
  (
   cellml_api_IRoleIterator* aRoleIt,
   nsIRDFContainer* aRoleSeq
  );
  void PopulateMathsIntoSequences
  (
   cellml_api_IMathMLElementIterator* aMEI,
   nsIRDFContainer* aMESeq
  );
  void PopulateEquationsIntoSequences
  (
   mathml_dom_IMathMLContainer* aEqContainer,
   PRUint32 aCount,
   nsIRDFContainer* aSeq
  );

  static PLDHashOperator rememberExpansionCallback(const void*,ResourceAssociation*,void*);
  void RememberExpansion(const nsIRDFResource*,ResourceAssociation*);
};
